@@ -1,4 +1,4 @@
-# Copyright 2009 10gen, Inc.
+# Copyright 2009-2010 10gen, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,9 +16,11 @@
 
 The :mod:`gridfs` package is an implementation of GridFS on top of
 :mod:`pymongo`, exposing a file-like interface.
+
+.. mongodoc:: gridfs
 """
 
-from grid_file import GridFile
+from gridfs.grid_file import GridFile
 from pymongo.database import Database
 
 class GridFS(object):
@@ -32,6 +34,8 @@ class GridFS(object):
 
         :Parameters:
           - `database`: database to use
+
+        .. mongodoc:: gridfs
         """
         if not isinstance(database, Database):
             raise TypeError("database must be an instance of Database")
