@@ -1,4 +1,4 @@
-# Copyright 2009 10gen, Inc.
+# Copyright 2009-2010 10gen, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,19 +17,19 @@
 import datetime
 import math
 import os
-from threading import Condition
 try:
     from cStringIO import StringIO
 except:
     from StringIO import StringIO
+from threading import Condition
 
-from pymongo.son import SON
-from pymongo.database import Database
-from pymongo.objectid import ObjectId
-from pymongo.dbref import DBRef
-from pymongo.binary import Binary
 from gridfs.errors import CorruptGridFile
 from pymongo import ASCENDING
+from pymongo.binary import Binary
+from pymongo.database import Database
+from pymongo.dbref import DBRef
+from pymongo.objectid import ObjectId
+from pymongo.son import SON
 
 try:
     _SEEK_SET = os.SEEK_SET
@@ -164,17 +164,17 @@ class GridFile(object):
 
         self.__collection.chunks.remove({"files_id": self.__id})
 
+    @property
     def closed(self):
         """Is this :class:`GridFile` closed?
         """
         return self.__closed
-    closed = property(closed)
 
+    @property
     def mode(self):
         """Mode this :class:`GridFile` was opened with.
         """
         return self.__mode
-    mode = property(mode)
 
     def __create_property(field_name, docstring, read_only=False):
         def getter(self):
@@ -306,7 +306,7 @@ class GridFile(object):
             size = remainder
 
         bytes = self.__buffer
-        chunk_number = math.floor(self.__position / self.__chunk_size)
+        chunk_number = (len(bytes) + self.__position) / self.__chunk_size
 
         while len(bytes) < size:
             chunk = self.__collection.chunks.find_one({"files_id": self.__id, "n": chunk_number})
