@@ -1,4 +1,4 @@
-# Copyright 2009 10gen, Inc.
+# Copyright 2009-2010 10gen, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -28,8 +28,19 @@ SLOW_ONLY = 1
 ALL = 2
 """Profile all operations."""
 
-version = "1.4"
+version = "1.5"
 """Current version of PyMongo."""
 
 Connection = PyMongo_Connection
 """Alias for :class:`pymongo.connection.Connection`."""
+
+def has_c():
+    """Is the C extension installed?
+
+    .. versionadded:: 1.5
+    """
+    try:
+        from pymongo import _cbson
+        return True
+    except ImportError:
+        return False
