@@ -1,5 +1,5 @@
 /*
- * Copyright 2009 10gen, Inc.
+ * Copyright 2009-2010 10gen, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -43,6 +43,7 @@ static PyObject* ObjectId;
 static PyObject* DBRef;
 static PyObject* RECompile;
 static PyObject* UUID;
+static PyObject* Timestamp;
 
 #if PY_VERSION_HEX < 0x02050000 && !defined(PY_SSIZE_T_MIN)
 typedef int Py_ssize_t;
@@ -462,6 +463,32 @@ static int write_element_to_buffer(bson_buffer* buffer, int type_byte, PyObject*
         Py_DECREF(as_doc);
         *(buffer->buffer + type_byte) = 0x03;
         return 1;
+    } else if (PyObject_IsInstance(value, Timestamp)) {
+        PyObject* obj;
+        long i;
+        
+        obj = PyObject_GetAttrString(value, "inc");
+        if (!obj) {
+            return 0;
+        }
+        i = PyInt_AsLong(obj);
+        Py_DECREF(obj);
+        if (!buffer_write_bytes(buffer, (const char*)&i, 4)) {
+            return 0;
+        }
+
+        obj = PyObject_GetAttrString(value, "time");
+        if (!obj) {
+            return 0;
+        }
+        i = PyInt_AsLong(obj);
+        Py_DECREF(obj);
+        if (!buffer_write_bytes(buffer, (const char*)&i, 4)) {
+            return 0;
+        }
+
+        *(buffer->buffer + type_byte) = 0x11;
+        return 1;
     }
     else if (PyObject_HasAttrString(value, "pattern") &&
              PyObject_HasAttrString(value, "flags")) { /* TODO just a proxy for checking if it is a compiled re */
@@ -580,6 +607,13 @@ static int write_element_to_buffer(bson_buffer* buffer, int type_byte, PyObject*
         DBRef = PyObject_GetAttrString(module, "DBRef");
         Py_DECREF(module);
 
+        module = PyImport_ImportModule("pymongo.timestamp");
+        if (!module) {
+            return 0;
+        }
+        Timestamp = PyObject_GetAttrString(module, "Timestamp");
+        Py_DECREF(module);
+
         module = PyImport_ImportModule("uuid");
         if (!module) {
             UUID = NULL;
@@ -593,12 +627,13 @@ static int write_element_to_buffer(bson_buffer* buffer, int type_byte, PyObject*
             PyObject_IsInstance(value, Code) ||
             PyObject_IsInstance(value, ObjectId) ||
             PyObject_IsInstance(value, DBRef) ||
+            PyObject_IsInstance(value, Timestamp) ||
             (UUID && PyObject_IsInstance(value, UUID))) {
 
             PyErr_SetString(PyExc_RuntimeError,
                             "A python module was reloaded without the C extension being reloaded.\n"
                             "\n"
-                            "See http://www.mongodb.org/display/DOCS/PyMongo+and+mod_wsgi for"
+                            "See http://api.mongodb.org/python/current/faq.html#does-pymongo-work-with-mod-wsgi for "
                             "a possible explanation / fix.");
             return 0;
         }
@@ -1003,6 +1038,121 @@ static PyObject* _cbson_update_message(PyObject* self, PyObject* args) {
     return result;
 }
 
+static PyObject* _cbson_query_message(PyObject* self, PyObject* args) {
+    /* NOTE just using a random number as the request_id */
+    int request_id = rand();
+    unsigned int options;
+    char* collection_name = NULL;
+    int collection_name_length;
+    int num_to_skip;
+    int num_to_return;
+    PyObject* query;
+    PyObject* field_selector = Py_None;
+    bson_buffer* buffer;
+    int length_location;
+    PyObject* result;
+
+    if (!PyArg_ParseTuple(args, "Iet#iiO|O",
+                          &options,
+                          "utf-8",
+                          &collection_name,
+                          &collection_name_length,
+                          &num_to_skip, &num_to_return,
+                          &query, &field_selector)) {
+        return NULL;
+    }
+    buffer = buffer_new();
+    if (!buffer) {
+        PyMem_Free(collection_name);
+        return NULL;
+    }
+
+    // save space for message length
+    length_location = buffer_save_bytes(buffer, 4);
+    if (length_location == -1 ||
+        !buffer_write_bytes(buffer, (const char*)&request_id, 4) ||
+        !buffer_write_bytes(buffer,
+                            "\x00\x00\x00\x00"
+                            "\xd4\x07\x00\x00", 8) ||
+        !buffer_write_bytes(buffer, (const char*)&options, 4) ||
+        !buffer_write_bytes(buffer,
+                            collection_name,
+                            collection_name_length + 1) ||
+        !buffer_write_bytes(buffer, (const char*)&num_to_skip, 4) ||
+        !buffer_write_bytes(buffer, (const char*)&num_to_return, 4) ||
+        !write_dict(buffer, query, 0, 1) ||
+        ((field_selector != Py_None) &&
+         !write_dict(buffer, field_selector, 0, 1))) {
+        buffer_free(buffer);
+        PyMem_Free(collection_name);
+        return NULL;
+    }
+
+    PyMem_Free(collection_name);
+
+    memcpy(buffer->buffer + length_location, &buffer->position, 4);
+
+    /* objectify buffer */
+    result = Py_BuildValue("is#", request_id,
+                           buffer->buffer, buffer->position);
+    buffer_free(buffer);
+    return result;
+}
+
+static PyObject* _cbson_get_more_message(PyObject* self, PyObject* args) {
+    /* NOTE just using a random number as the request_id */
+    int request_id = rand();
+    char* collection_name = NULL;
+    int collection_name_length;
+    int num_to_return;
+    long long cursor_id;
+    bson_buffer* buffer;
+    int length_location;
+    PyObject* result;
+
+    if (!PyArg_ParseTuple(args, "et#iL",
+                          "utf-8",
+                          &collection_name,
+                          &collection_name_length,
+                          &num_to_return,
+                          &cursor_id)) {
+        return NULL;
+    }
+    buffer = buffer_new();
+    if (!buffer) {
+        PyMem_Free(collection_name);
+        return NULL;
+    }
+
+    // save space for message length
+    length_location = buffer_save_bytes(buffer, 4);
+    if (length_location == -1 ||
+        !buffer_write_bytes(buffer, (const char*)&request_id, 4) ||
+        !buffer_write_bytes(buffer,
+                            "\x00\x00\x00\x00"
+                            "\xd5\x07\x00\x00"
+                            "\x00\x00\x00\x00", 12) ||
+        !buffer_write_bytes(buffer,
+                            collection_name,
+                            collection_name_length + 1) ||
+        !buffer_write_bytes(buffer, (const char*)&num_to_return, 4) ||
+        !buffer_write_bytes(buffer, (const char*)&cursor_id, 8)) {
+        buffer_free(buffer);
+        PyMem_Free(collection_name);
+        return NULL;
+    }
+
+    PyMem_Free(collection_name);
+
+    memcpy(buffer->buffer + length_location, &buffer->position, 4);
+
+    /* objectify buffer */
+    result = Py_BuildValue("is#", request_id,
+                           buffer->buffer, buffer->position);
+    buffer_free(buffer);
+    return result;
+}
+
 static PyObject* get_value(const char* buffer, int* position, int type) {
     PyObject* value;
     switch (type) {
@@ -1288,11 +1438,11 @@ static PyObject* get_value(const char* buffer, int* position, int type) {
         }
     case 17:
         {
-            int i,
-                j;
-            memcpy(&i, buffer + *position, 4);
-            memcpy(&j, buffer + *position + 4, 4);
-            value = Py_BuildValue("(ii)", i, j);
+            int time,
+                inc;
+            memcpy(&inc, buffer + *position, 4);
+            memcpy(&time, buffer + *position + 4, 4);
+            value = PyObject_CallFunction(Timestamp, "ii", time, inc);
             if (!value) {
                 return NULL;
             }
@@ -1424,6 +1574,10 @@ static PyMethodDef _CBSONMethods[] = {
      "create an insert message to be sent to MongoDB"},
     {"_update_message", _cbson_update_message, METH_VARARGS,
      "create an update message to be sent to MongoDB"},
+    {"_query_message", _cbson_query_message, METH_VARARGS,
+     "create a query message to be sent to MongoDB"},
+    {"_get_more_message", _cbson_get_more_message, METH_VARARGS,
+     "create a get more message to be sent to MongoDB"},
     {NULL, NULL, 0, NULL}
 };
 
@@ -1481,6 +1635,13 @@ PyMODINIT_FUNC init_cbson(void) {
     DBRef = PyObject_GetAttrString(module, "DBRef");
     Py_DECREF(module);
 
+    module = PyImport_ImportModule("pymongo.timestamp");
+    if (!module) {
+        return;
+    }
+    Timestamp = PyObject_GetAttrString(module, "Timestamp");
+    Py_DECREF(module);
+
     module = PyImport_ImportModule("re");
     if (!module) {
         return;
