@@ -1,4 +1,4 @@
-# Copyright 2009 10gen, Inc.
+# Copyright 2009-2010 10gen, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,13 +17,13 @@
 import warnings
 import struct
 
-import helpers
-import message
-from objectid import ObjectId
-from cursor import Cursor
-from son import SON
-from errors import InvalidName
-from code import Code
+from pymongo import (helpers,
+                     message)
+from pymongo.code import Code
+from pymongo.cursor import Cursor
+from pymongo.errors import InvalidName
+from pymongo.objectid import ObjectId
+from pymongo.son import SON
 
 _ZERO = "\x00\x00\x00\x00"
 
@@ -32,29 +32,50 @@ class Collection(object):
     """A Mongo collection.
     """
 
-    def __init__(self, database, name, options=None):
+    def __init__(self, database, name, options=None, create=False, **kwargs):
         """Get / create a Mongo collection.
 
         Raises :class:`TypeError` if `name` is not an instance of
         :class:`basestring`. Raises
         :class:`~pymongo.errors.InvalidName` if `name` is not a valid
-        collection name. Raises :class:`TypeError` if `options` is not
-        an instance of :class:`dict`. If `options` is non-empty a
-        create command will be sent to the database. Otherwise the
-        collection will be created implicitly on first use.
+        collection name. Any additional keyword arguments will be used
+        as options passed to the create command. See
+        :meth:`~pymongo.database.Database.create_collection` for valid
+        options.
+
+        If `create` is ``True`` or additional keyword arguments are
+        present a create command will be sent. Otherwise, a create
+        command will not be sent and the collection will be created
+        implicitly on first use.
 
         :Parameters:
           - `database`: the database to get a collection from
           - `name`: the name of the collection to get
-          - `options`: dictionary of collection options.  see
-            :meth:`~pymongo.database.Database.create_collection` for
-            details.
+          - `options`: DEPRECATED dictionary of collection options
+          - `create` (optional): if ``True``, force collection
+            creation even without options being set
+          - `**kwargs` (optional): additional keyword arguments will
+            be passed as options for the create collection command
+
+        .. versionchanged:: 1.5
+           deprecating `options` in favor of kwargs
+        .. versionadded:: 1.5
+           the `create` parameter
+
+        .. mongodoc:: collections
         """
         if not isinstance(name, basestring):
             raise TypeError("name must be an instance of basestring")
 
-        if options is not None and not isinstance(options, dict):
-            raise TypeError("options must be an instance of dict")
+        if options is not None:
+            warnings.warn("the options argument to Collection is deprecated "
+                          "and will be removed. please use kwargs instead.",
+                          DeprecationWarning)
+            if not isinstance(options, dict):
+                raise TypeError("options must be an instance of dict")
+            options.update(kwargs)
+        elif kwargs:
+            options = kwargs
 
         if not name or ".." in name:
             raise InvalidName("collection names cannot be empty")
@@ -79,7 +100,7 @@ class Collection(object):
                                                "Collection.name")
         self.__full_name_w = helpers.callable_value(self.__full_name,
                                                     "Collection.full_name")
-        if options is not None:
+        if create or options is not None:
             self.__create(options)
 
     def __create(self, options):
@@ -88,7 +109,7 @@ class Collection(object):
 
         # Send size as a float, not an int/long. BSON can only handle 32-bit
         # ints which conflicts w/ max collection size of 10000000000.
-        if "size" in options:
+        if options and "size" in options:
             options["size"] = float(options["size"])
 
         command = SON({"create": self.__name})
@@ -118,6 +139,7 @@ class Collection(object):
                        (other.__database, other.__name))
         return NotImplemented
 
+    @property
     def full_name(self):
         """The full name of this :class:`Collection`.
 
@@ -128,8 +150,8 @@ class Collection(object):
            ``full_name()`` method is deprecated.
         """
         return self.__full_name_w
-    full_name = property(full_name)
 
+    @property
     def name(self):
         """The name of this :class:`Collection`.
 
@@ -138,8 +160,8 @@ class Collection(object):
            ``name()`` method is deprecated.
         """
         return self.__name_w
-    name = property(name)
 
+    @property
     def database(self):
         """The :class:`~pymongo.database.Database` that this
         :class:`Collection` is a part of.
@@ -149,7 +171,6 @@ class Collection(object):
            ``database()`` method is deprecated.
         """
         return self.__database_w
-    database = property(database)
 
     def save(self, to_save, manipulate=True, safe=False):
         """Save a document in this collection.
@@ -169,6 +190,8 @@ class Collection(object):
           - `to_save`: the SON object to be saved
           - `manipulate` (optional): manipulate the SON object before saving it
           - `safe` (optional): check that the save succeeded?
+
+        .. mongodoc:: insert
         """
         if not isinstance(to_save, dict):
             raise TypeError("cannot save object of type %s" % type(to_save))
@@ -201,9 +224,13 @@ class Collection(object):
 
         .. versionchanged:: 1.1
            Bulk insert works with any iterable
+
+        .. mongodoc:: insert
         """
         docs = doc_or_docs
+        return_one = False
         if isinstance(docs, dict):
+            return_one = True
             docs = [docs]
 
         if manipulate:
@@ -213,21 +240,25 @@ class Collection(object):
             message.insert(self.__full_name, docs, check_keys, safe), safe)
 
         ids = [doc.get("_id", None) for doc in docs]
-        return len(ids) == 1 and ids[0] or ids
+        return return_one and ids[0] or ids
 
     def update(self, spec, document,
                upsert=False, manipulate=False, safe=False, multi=False):
         """Update a document(s) in this collection.
 
-        Raises :class:`TypeError` if either `spec` or `document` is not an
-        instance of ``dict`` or `upsert` is not an instance of ``bool``. If
-        `safe` is ``True`` then the update will be checked for errors, raising
-        :class:`~pymongo.errors.OperationFailure` if one occurred. Safe inserts
-        wait for a response from the database, while normal inserts do not.
+        Raises :class:`TypeError` if either `spec` or `document` is
+        not an instance of ``dict`` or `upsert` is not an instance of
+        ``bool``. If `safe` is ``True`` then the update will be
+        checked for errors, raising
+        :class:`~pymongo.errors.OperationFailure` if one
+        occurred. Safe updates require a response from the database,
+        while normal updates do not - thus, setting `safe` to ``True``
+        will negatively impact performance.
 
-        There are many useful `update modifiers`_ which can be used when
-        performing updates. For example, here we use the ``"$set"`` modifier to
-        modify some fields in a matching document:
+        There are many useful `update modifiers`_ which can be used
+        when performing updates. For example, here we use the
+        ``"$set"`` modifier to modify some fields in a matching
+        document:
 
         .. doctest::
 
@@ -243,20 +274,26 @@ class Collection(object):
         command. Otherwise, returns ``None``.
 
         :Parameters:
-          - `spec`: a ``dict`` or :class:`~pymongo.son.SON` instance specifying
-            elements which must be present for a document to be updated
-          - `document`: a ``dict`` or :class:`~pymongo.son.SON` instance
-            specifying the document to be used for the update or (in the case
-            of an upsert) insert - see docs on MongoDB `update modifiers`_
-          - `upsert` (optional): perform an upsert operation
-          - `manipulate` (optional): manipulate the document before updating?
+          - `spec`: a ``dict`` or :class:`~pymongo.son.SON` instance
+            specifying elements which must be present for a document
+            to be updated
+          - `document`: a ``dict`` or :class:`~pymongo.son.SON`
+            instance specifying the document to be used for the update
+            or (in the case of an upsert) insert - see docs on MongoDB
+            `update modifiers`_
+          - `upsert` (optional): perform an `upsert`_ if ``True``
+          - `manipulate` (optional): manipulate the document before
+            updating? If ``True`` all instances of
+            :mod:`~pymongo.son_manipulator.SONManipulator` added to
+            this :class:`~pymongo.database.Database` will be applied
+            to the document before performing the update.
           - `safe` (optional): check that the update succeeded?
-          - `multi` (optional): update all documents that match `spec`, rather
-            than just the first matching document. The default value for
-            `multi` is currently ``False``, but this might eventually change to
-            ``True``. It is recommended that you specify this argument
-            explicitly for all update operations in order to prepare your code
-            for that change.
+          - `multi` (optional): update all documents that match
+            `spec`, rather than just the first matching document. The
+            default value for `multi` is currently ``False``, but this
+            might eventually change to ``True``. It is recommended
+            that you specify this argument explicitly for all update
+            operations in order to prepare your code for that change.
 
         .. versionchanged:: 1.4
            Return the response to *lastError* if `safe` is ``True``.
@@ -264,6 +301,9 @@ class Collection(object):
            The `multi` parameter.
 
         .. _update modifiers: http://www.mongodb.org/display/DOCS/Updating
+        .. _upsert: http://www.mongodb.org/display/DOCS/Updating#Updating-Upserts
+
+        .. mongodoc:: update
         """
         if not isinstance(spec, dict):
             raise TypeError("spec must be an instance of dict")
@@ -317,6 +357,8 @@ class Collection(object):
            removed.
         .. versionadded:: 1.1
            The `safe` parameter.
+
+        .. mongodoc:: remove
         """
         spec = spec_or_object_id
         if spec is None:
@@ -421,6 +463,8 @@ class Collection(object):
 
         .. versionadded:: 1.1
            The `tailable` parameter.
+
+        .. mongodoc:: find
         """
         if spec is None:
             spec = SON()
@@ -465,7 +509,7 @@ class Collection(object):
         """
         return u"_".join([u"%s_%s" % item for item in keys])
 
-    def create_index(self, key_or_list, unique=False, ttl=300):
+    def create_index(self, key_or_list, unique=False, ttl=300, name=None):
         """Creates an index on this collection.
 
         Takes either a single key or a list of (key, direction) pairs.
@@ -474,6 +518,17 @@ class Collection(object):
         :data:`~pymongo.DESCENDING`). Returns the name of the created
         index.
 
+        To create a single key index on the key ``'mike'`` we just use
+        a string argument:
+
+        >>> my_collection.create_index("mike")
+
+        For a `compound index`_ on ``'mike'`` descending and
+        ``'eliot'`` ascending we need to use a list of tuples:
+
+        >>> my_collection.create_index([("mike", pymongo.DESCENDING),
+        ...                             ("eliot", pymongo.ASCENDING)])
+
         :Parameters:
           - `key_or_list`: a single key or a list of (key, direction)
             pairs specifying the index to create
@@ -483,17 +538,25 @@ class Collection(object):
             this index will be recognized by subsequent calls to
             :meth:`ensure_index` - see documentation for
             :meth:`ensure_index` for details
-        """
-        if not isinstance(key_or_list, (str, unicode, list)):
-            raise TypeError("key_or_list must either be a single key "
-                            "or a list of (key, direction) pairs")
+          - `name` (optional): name for the index. If none given, a name
+            will be generated.
 
-        to_save = SON()
+        .. versionadded:: 1.5
+           The `name` parameter.
+
+        .. seealso:: :meth:`ensure_index`
+
+        .. _compound index: http://www.mongodb.org/display/DOCS/Indexes#Indexes-CompoundKeysIndexes
+
+        .. mongodoc:: indexes
+        """
         keys = helpers._index_list(key_or_list)
-        name = self._gen_index_name(keys)
+        index_doc = helpers._index_document(keys)
+        name = name is not None and name or self._gen_index_name(keys)
+        to_save = SON()
         to_save["name"] = name
         to_save["ns"] = self.__full_name
-        to_save["key"] = helpers._index_document(keys)
+        to_save["key"] = index_doc
         to_save["unique"] = unique
 
         self.__database.connection._cache_index(self.__database.name,
@@ -503,13 +566,14 @@ class Collection(object):
                                               check_keys=False)
         return to_save["name"]
 
-    def ensure_index(self, key_or_list, unique=False, ttl=300):
+    def ensure_index(self, key_or_list, unique=False, ttl=300, name=None):
         """Ensures that an index exists on this collection.
 
         Takes either a single key or a list of (key, direction) pairs.
         The key(s) must be an instance of :class:`basestring`, and the
         direction(s) must be one of (:data:`~pymongo.ASCENDING`,
-        :data:`~pymongo.DESCENDING`).
+        :data:`~pymongo.DESCENDING`). See :meth:`create_index` for a
+        detailed example.
 
         Unlike :meth:`create_index`, which attempts to create an index
         unconditionally, :meth:`ensure_index` takes advantage of some
@@ -537,15 +601,23 @@ class Collection(object):
           - `ttl` (optional): time window (in seconds) during which
             this index will be recognized by subsequent calls to
             :meth:`ensure_index`
+          - `name` (optional): name for the index. If none given, a name
+            will be generated.
+
+        .. versionadded:: 1.5
+           The `name` parameter.
+
+        .. seealso:: :meth:`create_index`
         """
         if not isinstance(key_or_list, (str, unicode, list)):
             raise TypeError("key_or_list must either be a single key or a list of (key, direction) pairs")
 
         keys = helpers._index_list(key_or_list)
-        name = self._gen_index_name(keys)
+        name = name is not None and name or self._gen_index_name(keys)
         if self.__database.connection._cache_index(self.__database.name,
                                                    self.__name, name, ttl):
-            return self.create_index(key_or_list, unique=unique, ttl=ttl)
+            return self.create_index(key_or_list, unique=unique, ttl=ttl,
+                                     name=name)
         return None
 
     def drop_indexes(self):
@@ -561,12 +633,16 @@ class Collection(object):
     def drop_index(self, index_or_name):
         """Drops the specified index on this collection.
 
-        Can be used on non-existant collections or collections with no indexes.
-        Raises OperationFailure on an error. `index_or_name` can be either an
-        index name (as returned by `create_index`), or an index specifier (as
-        passed to `create_index`). An index specifier should be a list of (key,
-        direction) pairs. Raises TypeError if index is not an instance of (str,
-        unicode, list).
+        Can be used on non-existant collections or collections with no
+        indexes.  Raises OperationFailure on an error. `index_or_name`
+        can be either an index name (as returned by `create_index`),
+        or an index specifier (as passed to `create_index`). An index
+        specifier should be a list of (key, direction) pairs. Raises
+        TypeError if index is not an instance of (str, unicode, list).
+
+        .. warning:: if a custom name was used on index creation (by
+          passing the `name` parameter to :meth:`create_index` or
+          :meth:`ensure_index`) the index **must** be dropped by name.
 
         :Parameters:
           - `index_or_name`: index (or name of index) to drop
@@ -602,9 +678,9 @@ class Collection(object):
         """Get the options set on this collection.
 
         Returns a dictionary of options and their values - see
-        `pymongo.database.Database.create_collection` for more information on
-        the options dictionary. Returns an empty dictionary if the collection
-        has not been created yet.
+        :meth:`~pymongo.database.Database.create_collection` for more
+        information on the possible options. Returns an empty
+        dictionary if the collection has not been created yet.
         """
         result = self.__database.system.namespaces.find_one(
             {"name": self.__full_name})
@@ -743,6 +819,8 @@ class Collection(object):
         .. versionadded:: 1.2
 
         .. _map reduce command: http://www.mongodb.org/display/DOCS/MapReduce
+
+        .. mongodoc:: mapreduce
         """
         command = SON([("mapreduce", self.__name),
                        ("map", map), ("reduce", reduce)])
