@@ -1,4 +1,4 @@
-# Copyright 2009 10gen, Inc.
+# Copyright 2009-2010 10gen, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -33,22 +33,28 @@ To get a :class:`~pymongo.database.Database` instance from a
   Database(Connection('localhost', 27017), u'test-database')
 """
 
-import sys
+import datetime
+import errno
+import os
+import random
 import socket
 import struct
+import sys
 import threading
-import random
-import errno
-import datetime
 import warnings
 
-from errors import ConnectionFailure, ConfigurationError, AutoReconnect
-from errors import OperationFailure, DuplicateKeyError
-from database import Database
-from cursor_manager import CursorManager
-import bson
-import message
-import helpers
+from pymongo import (bson,
+                     database,
+                     helpers,
+                     message)
+from pymongo.cursor_manager import CursorManager
+from pymongo.errors import (AutoReconnect,
+                            ConfigurationError,
+                            ConnectionFailure,
+                            DuplicateKeyError,
+                            InvalidURI,
+                            OperationFailure)
+from pymongo.son import SON
 
 _CONNECT_TIMEOUT = 20.0
 
@@ -70,18 +76,25 @@ class Pool(threading.local):
         self.socket_factory = socket_factory
 
     def socket(self):
-        if self.sock is not None:
-            return self.sock
+        # we store the pid here to avoid issues with fork /
+        # multiprocessing - see
+        # test.test_connection:TestConnection.test_fork for an example
+        # of what could go wrong otherwise
+        pid = os.getpid()
+
+        if self.sock is not None and self.sock[0] == pid:
+            return self.sock[1]
 
         try:
-            self.sock = self.sockets.pop()
+            self.sock = (pid, self.sockets.pop())
         except IndexError:
-            self.sock = self.socket_factory()
-        return self.sock
+            self.sock = (pid, self.socket_factory())
+
+        return self.sock[1]
 
     def return_socket(self):
-        if self.sock is not None:
-            self.sockets.append(self.sock)
+        if self.sock is not None and self.sock[0] == os.getpid():
+            self.sockets.append(self.sock[1])
         self.sock = None
 
 
@@ -129,6 +142,8 @@ class Connection(object): # TODO support auth for pooling
            parameters.
         .. versionadded:: 1.1
            The `network_timeout` parameter.
+
+        .. mongodoc:: connections
         """
         if host is None:
             host = self.HOST
@@ -168,6 +183,90 @@ class Connection(object): # TODO support auth for pooling
         if _connect:
             self.__find_master()
 
+    @staticmethod
+    def __partition(source, sub):
+        i = source.find(sub)
+        if i == -1:
+            return (source, None)
+        return (source[:i], source[i+len(sub):])
+
+    @staticmethod
+    def _parse_uri(uri):
+        info = {}
+
+        if uri.startswith("mongodb://"):
+            uri = uri[len("mongodb://"):]
+        elif "://" in uri:
+            raise InvalidURI("Invalid uri scheme: %s" % Connection.__partition(uri, "://")[0])
+
+        (hosts, database) = Connection.__partition(uri, "/")
+
+        if not database:
+            database = None
+
+        username = None
+        password = None
+        if "@" in hosts:
+            (auth, hosts) = Connection.__partition(hosts, "@")
+
+            if ":" not in auth:
+                raise InvalidURI("auth must be specified as 'username:password@'")
+            (username, password) = Connection.__partition(auth, ":")
+
+        host_list = []
+        for host in hosts.split(","):
+            if not host:
+                raise InvalidURI("empty host (or extra comma in host list)")
+            (hostname, port) = Connection.__partition(host, ":")
+            if port:
+                port = int(port)
+            else:
+                port = 27017
+            host_list.append((hostname, port))
+
+        return (host_list, database, username, password)
+
+    @classmethod
+    def from_uri(cls, uri="mongodb://localhost", **connection_args):
+        """Connect to a MongoDB instance(s) using the mongodb URI
+        scheme.
+
+        The format for a MongoDB URI is documented `here
+        <http://dochub.mongodb.org/core/connections>`_. Raises
+        :class:`~pymongo.errors.InvalidURI` when given an invalid URI.
+
+        :Parameters:
+
+          - `uri`: URI identifying the MongoDB instance(s) to connect
+            to
+
+        The remaining keyword arguments are the same as those accepted
+        by :meth:`~Connection`.
+
+        .. versionadded:: 1.5
+        """
+        (nodes, database, username, password) = Connection._parse_uri(uri)
+        if database and username is None:
+            raise InvalidURI("cannot specify database without "
+                             "a username and password")
+
+        if len(nodes) == 1:
+            connection = cls(*nodes[0], **connection_args)
+
+        elif len(nodes) == 2:
+            connection = cls.paired(*nodes, **connection_args)
+
+        else:
+            raise InvalidURI("Connecting to more than 2 nodes "
+                             "is not currently supported")
+
+        if username:
+            database = database or "admin"
+            if not connection[database].authenticate(username, password):
+                raise InvalidURI("authentication failed")
+
+        return connection
+
     def __pair_with(self, host, port):
         """Pair this connection with a Mongo instance running on host:port.
 
@@ -188,8 +287,8 @@ class Connection(object): # TODO support auth for pooling
 
         self.__find_master()
 
-    def paired(cls, left, right=None,
-               pool_size=None, auto_start_request=None):
+    @classmethod
+    def paired(cls, left, right=None, **connection_args):
         """Open a new paired connection to Mongo.
 
         Raises :class:`TypeError` if either `left` or `right` is not a tuple of
@@ -200,30 +299,30 @@ class Connection(object): # TODO support auth for pooling
           - `left`: ``(host, port)`` pair for the left MongoDB instance
           - `right` (optional): ``(host, port)`` pair for the right MongoDB
             instance
-          - `pool_size` (optional): DEPRECATED
-          - `auto_start_request` (optional): DEPRECATED
+
+        The remaining keyword arguments are the same as those accepted
+        by :meth:`~Connection`.
         """
         if right is None:
             right = (cls.HOST, cls.PORT)
-        if pool_size is not None:
-            warnings.warn("The pool_size parameter to Connection.paired is "
-                          "deprecated", DeprecationWarning)
-        if auto_start_request is not None:
-            warnings.warn("The auto_start_request parameter to "
-                          "Connection.paired is deprecated",
-                          DeprecationWarning)
 
-        connection = cls(left[0], left[1], _connect=False)
+        for param in ('pool_size', 'auto_start_request', 'timeout'):
+            if param in connection_args:
+                warnings.warn("The %s parameter to Connection.paired is "
+                              "deprecated" % param, DeprecationWarning)
+
+        connection_args['_connect'] = False
+
+        connection = cls(left[0], left[1], **connection_args)
         connection.__pair_with(*right)
         return connection
-    paired = classmethod(paired)
 
     def __master(self, sock):
         """Get the hostname and port of the master Mongo instance.
 
         Return a tuple (host, port).
         """
-        result = self["admin"].command({"ismaster": 1}, _sock=sock)
+        result = self["admin"].command("ismaster", _sock=sock)
 
         if result["ismaster"] == 1:
             return True
@@ -292,6 +391,7 @@ class Connection(object): # TODO support auth for pooling
         if index_name in self.__index_cache[database_name][collection_name]:
             del self.__index_cache[database_name][collection_name][index_name]
 
+    @property
     def host(self):
         """Current connected host.
 
@@ -300,8 +400,8 @@ class Connection(object): # TODO support auth for pooling
            method is deprecated.
         """
         return helpers.callable_value(self.__host, "Connection.host")
-    host = property(host)
 
+    @property
     def port(self):
         """Current connected port.
 
@@ -310,13 +410,12 @@ class Connection(object): # TODO support auth for pooling
            method is deprecated.
         """
         return helpers.callable_value(self.__port, "Connection.port")
-    port = property(port)
 
+    @property
     def slave_okay(self):
         """Is it okay for this connection to connect directly to a slave?
         """
         return self.__slave_okay
-    slave_okay = property(slave_okay)
 
     def __find_master(self):
         """Create a new socket and use it to figure out who the master is.
@@ -588,17 +687,19 @@ class Connection(object): # TODO support auth for pooling
     def __getattr__(self, name):
         """Get a database by name.
 
-        Raises InvalidName if an invalid database name is used.
+        Raises :class:`~pymongo.errors.InvalidName` if an invalid
+        database name is used.
 
         :Parameters:
           - `name`: the name of the database to get
         """
-        return Database(self, name)
+        return database.Database(self, name)
 
     def __getitem__(self, name):
         """Get a database by name.
 
-        Raises InvalidName if an invalid database name is used.
+        Raises :class:`~pymongo.errors.InvalidName` if an invalid
+        database name is used.
 
         :Parameters:
           - `name`: the name of the database to get
@@ -608,7 +709,9 @@ class Connection(object): # TODO support auth for pooling
     def close_cursor(self, cursor_id):
         """Close a single database cursor.
 
-        Raises :class:`TypeError` if `cursor_id` is not an instance of ``(int, long)``. What closing the cursor actually means depends on this connection's cursor manager.
+        Raises :class:`TypeError` if `cursor_id` is not an instance of
+        ``(int, long)``. What closing the cursor actually means
+        depends on this connection's cursor manager.
 
         :Parameters:
           - `cursor_id`: id of cursor to close
@@ -637,14 +740,14 @@ class Connection(object): # TODO support auth for pooling
     def __database_info(self):
         """Get a dictionary of (database_name: size_on_disk).
         """
-        result = self["admin"].command({"listDatabases": 1})
+        result = self["admin"].command("listDatabases")
         info = result["databases"]
         return dict([(db["name"], db["sizeOnDisk"]) for db in info])
 
     def server_info(self):
         """Get information about the MongoDB server we're connected to.
         """
-        return self.admin.command({"buildinfo": 1})
+        return self.admin.command("buildinfo")
 
     def database_names(self):
         """Get a list of the names of all databases on the connected server.
@@ -663,7 +766,7 @@ class Connection(object): # TODO support auth for pooling
             database to drop
         """
         name = name_or_database
-        if isinstance(name, Database):
+        if isinstance(name, database.Database):
             name = name.name
 
         if not isinstance(name, basestring):
@@ -671,7 +774,52 @@ class Connection(object): # TODO support auth for pooling
                             "(Database, str, unicode)")
 
         self._purge_index(name)
-        self[name].command({"dropDatabase": 1})
+        self[name].command("dropDatabase")
+
+    def copy_database(self, from_name, to_name,
+                      from_host=None, username=None, password=None):
+        """Copy a database, potentially from another host.
+
+        Raises :class:`TypeError` if `from_name` or `to_name` is not
+        an instance of :class:`basestring`. Raises
+        :class:`~pymongo.errors.InvalidName` if `to_name` is not a
+        valid database name.
+
+        If `from_host` is ``None`` the current host is used as the
+        source. Otherwise the database is copied from `from_host`.
+
+        If the source database requires authentication, `username` and
+        `password` must be specified.
+
+        :Parameters:
+          - `from_name`: the name of the source database
+          - `to_name`: the name of the target database
+          - `from_host` (optional): host name to copy from
+          - `username` (optional): username for source database
+          - `password` (optional): password for source database
+
+        .. versionadded:: 1.5
+        """
+        if not isinstance(from_name, basestring):
+            raise TypeError("from_name must be an instance of basestring")
+        if not isinstance(to_name, basestring):
+            raise TypeError("to_name must be an instance of basestring")
+
+        database._check_name(to_name)
+
+        command = SON([("copydb", 1), ("fromdb", from_name), ("todb", to_name)])
+
+        if from_host is not None:
+            command["fromhost"] = from_host
+
+        if username is not None:
+            nonce = self.admin.command(SON([("copydbgetnonce", 1),
+                                            ("fromhost", from_host)]))["nonce"]
+            command["username"] = username
+            command["nonce"] = nonce
+            command["key"] = helpers._auth_key(nonce, username, password)
+
+        return self.admin.command(command)
 
     def __iter__(self):
         return self
