@@ -1,4 +1,4 @@
-# Copyright 2009 10gen, Inc.
+# Copyright 2009-2010 10gen, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,11 +17,12 @@
 import struct
 import warnings
 
-import helpers
-import message
-from son import SON
-from code import Code
-from errors import InvalidOperation, AutoReconnect
+from pymongo import (helpers,
+                     message)
+from pymongo.code import Code
+from pymongo.errors import (InvalidOperation,
+                            AutoReconnect)
+from pymongo.son import SON
 
 _QUERY_OPTIONS = {
     "tailable_cursor": 2,
@@ -41,6 +42,8 @@ class Cursor(object):
         """Create a new cursor.
 
         Should not be called directly by application developers.
+
+        .. mongodoc:: cursors
         """
         self.__collection = collection
         self.__spec = spec
@@ -64,6 +67,7 @@ class Cursor(object):
         self.__retrieved = 0
         self.__killed = False
 
+    @property
     def collection(self):
         """The :class:`~pymongo.collection.Collection` that this
         :class:`Cursor` is iterating.
@@ -71,7 +75,6 @@ class Cursor(object):
         .. versionadded:: 1.1
         """
         return self.__collection
-    collection = property(collection)
 
     def __del__(self):
         if self.__id and not self.__killed:
@@ -125,11 +128,11 @@ class Cursor(object):
     def __query_spec(self):
         """Get the spec to use for a query.
         """
-        if self.__is_command:
+        if self.__is_command or "$query" in self.__spec:
             return self.__spec
-        spec = SON({"query": self.__spec})
+        spec = SON({"$query": self.__spec})
         if self.__ordering:
-            spec["orderby"] = self.__ordering
+            spec["$orderby"] = self.__ordering
         if self.__explain:
             spec["$explain"] = True
         if self.__hint:
@@ -165,6 +168,8 @@ class Cursor(object):
 
         :Parameters:
           - `limit`: the number of results to return
+
+        .. mongodoc:: limit
         """
         if not isinstance(limit, int):
             raise TypeError("limit must be an int")
@@ -306,7 +311,7 @@ class Cursor(object):
             if self.__skip:
                 command["skip"] = self.__skip
 
-        response = self.__collection.database.command(command, ["ns missing"])
+        response = self.__collection.database.command(command, allowable_errors=["ns missing"])
         if response.get("errmsg", "") == "ns missing":
             return 0
         return int(response["n"])
@@ -339,6 +344,8 @@ class Cursor(object):
 
     def explain(self):
         """Returns an explain plan record for this cursor.
+
+        .. mongodoc:: explain
         """
         c = self.clone()
         c.__explain = True
@@ -374,8 +381,6 @@ class Cursor(object):
             self.__hint = None
             return self
 
-        if not isinstance(index, (list)):
-            raise TypeError("hint takes a list specifying an index")
         self.__hint = helpers._index_document(index)
         return self
 
@@ -477,6 +482,19 @@ class Cursor(object):
 
         return len(self.__data)
 
+    @property
+    def alive(self):
+        """Does this cursor have the potential to return more data?
+
+        This is mostly useful with `tailable cursors
+        <http://www.mongodb.org/display/DOCS/Tailable+Cursors>`_
+        since they will stop iterating even though they *may* return more
+        results in the future.
+
+        .. versionadded:: 1.5
+        """
+        return bool(len(self.__data) or (not self.__killed))
+
     def __iter__(self):
         return self
 
