@@ -1,4 +1,4 @@
-# Copyright 2009 10gen, Inc.
+# Copyright 2009-2010 10gen, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,20 +15,29 @@
 """Database level operations."""
 
 import warnings
-try:
-    import hashlib
-    _md5func = hashlib.md5
-except: # for Python < 2.5
-    import md5
-    _md5func = md5.new
 
-from son import SON
-from dbref import DBRef
-from son_manipulator import ObjectIdInjector, ObjectIdShuffler
-from collection import Collection
-from errors import InvalidName, CollectionInvalid, OperationFailure
-from code import Code
-import helpers
+from pymongo import helpers
+from pymongo.code import Code
+from pymongo.collection import Collection
+from pymongo.dbref import DBRef
+from pymongo.errors import (CollectionInvalid,
+                            InvalidName,
+                            OperationFailure)
+from pymongo.son import SON
+from pymongo.son_manipulator import (ObjectIdInjector,
+                                     ObjectIdShuffler)
+
+
+def _check_name(name):
+    """Check if a database name is valid.
+    """
+    if not name:
+        raise InvalidName("database name cannot be the empty string")
+
+    for invalid_char in [" ", ".", "$", "/", "\\"]:
+        if invalid_char in name:
+            raise InvalidName("database names cannot contain the "
+                              "character %r" % invalid_char)
 
 
 class Database(object):
@@ -47,11 +56,13 @@ class Database(object):
           - `connection`: a :class:`~pymongo.connection.Connection`
             instance
           - `name`: database name
+
+        .. mongodoc:: databases
         """
         if not isinstance(name, basestring):
             raise TypeError("name must be an instance of basestring")
 
-        self.__check_name(name)
+        _check_name(name)
 
         self.__name = unicode(name)
         self.__connection = connection
@@ -64,14 +75,7 @@ class Database(object):
         self.__outgoing_manipulators = []
         self.__outgoing_copying_manipulators = []
         self.add_son_manipulator(ObjectIdInjector())
-
-    def __check_name(self, name):
-        for invalid_char in [" ", ".", "$", "/", "\\"]:
-            if invalid_char in name:
-                raise InvalidName("database names cannot contain the "
-                                  "character %r" % invalid_char)
-        if not name:
-            raise InvalidName("database name cannot be the empty string")
+        self.__system_js = SystemJS(self)
 
     def add_son_manipulator(self, manipulator):
         """Add a new son manipulator to this database.
@@ -96,6 +100,17 @@ class Database(object):
             if method_overwritten(manipulator, "transform_outgoing"):
                 self.__outgoing_manipulators.insert(0, manipulator)
 
+    @property
+    def system_js(self):
+        """A :class:`SystemJS` helper for this :class:`Database`.
+
+        See the documentation for :class:`SystemJS` for more details.
+
+        .. versionadded:: 1.5
+        """
+        return self.__system_js
+
+    @property
     def connection(self):
         """The :class:`~pymongo.connection.Connection` instance for this
         :class:`Database`.
@@ -105,8 +120,8 @@ class Database(object):
            ``connection()`` method is deprecated.
         """
         return self.__connection_w
-    connection = property(connection)
 
+    @property
     def name(self):
         """The name of this :class:`Database`.
 
@@ -115,7 +130,6 @@ class Database(object):
            ``name()`` method is deprecated.
         """
         return self.__name_w
-    name = property(name)
 
     def __cmp__(self, other):
         if isinstance(other, Database):
@@ -146,29 +160,46 @@ class Database(object):
         """
         return self.__getattr__(name)
 
-    def create_collection(self, name, options={}):
-        """Create a new collection in this database.
+    def create_collection(self, name, options=None, **kwargs):
+        """Create a new :class:`~pymongo.collection.Collection` in this
+        database.
 
-        Normally collection creation is automatic. This method should only if
-        you want to specify options on creation. CollectionInvalid is raised
-        if the collection already exists.
+        Normally collection creation is automatic. This method should
+        only be used to specify options on
+        creation. :class:`~pymongo.errors.CollectionInvalid` will be
+        raised if the collection already exists.
 
-        Options should be a dictionary, with any of the following options:
+        Options should be passed as keyword arguments to this
+        method. Any of the following options are valid:
 
-          - "size": desired initial size for the collection (in bytes). must be
-            less than or equal to 10000000000. For capped collections this size
-            is the max size of the collection.
+          - "size": desired initial size for the collection (in
+            bytes). must be less than or equal to 10000000000. For
+            capped collections this size is the max size of the
+            collection.
           - "capped": if True, this is a capped collection
           - "max": maximum number of objects if capped (optional)
 
         :Parameters:
           - `name`: the name of the collection to create
-          - `options` (optional): options to use on the new collection
+          - `options`: DEPRECATED options to use on the new collection
+          - `**kwargs` (optional): additional keyword arguments will
+            be passed as options for the create collection command
+
+        .. versionchanged:: 1.5
+           deprecating `options` in favor of kwargs
         """
+        opts = {"create": True}
+        if options is not None:
+            warnings.warn("the options argument to create_collection is "
+                          "deprecated and will be removed. please use "
+                          "kwargs instead.", DeprecationWarning)
+            opts.update(options)
+        opts.update(kwargs)
+
         if name in self.collection_names():
             raise CollectionInvalid("collection %s already exists" % name)
 
-        return Collection(self, name, options)
+        return Collection(self, name, **opts)
 
     def _fix_incoming(self, son, collection):
         """Apply manipulators to an incoming SON object before it gets stored.
@@ -190,9 +221,9 @@ class Database(object):
           - `son`: the son object coming out of the database
           - `collection`: the collection the son object was saved in
         """
-        for manipulator in helpers._reversed(self.__outgoing_manipulators):
+        for manipulator in reversed(self.__outgoing_manipulators):
             son = manipulator.transform_outgoing(son, collection)
-        for manipulator in helpers._reversed(self.__outgoing_copying_manipulators):
+        for manipulator in reversed(self.__outgoing_copying_manipulators):
             son = manipulator.transform_outgoing(son, collection)
         return son
 
@@ -204,17 +235,36 @@ class Database(object):
     def command(self, command, check=True, allowable_errors=[], _sock=None):
         """Issue a MongoDB command.
 
-        Send a command to the database and return the response.
+        Send command `command` to the database and return the
+        response. If `command` is an instance of :class:`str` then the
+        command ``{command: 1}`` will be sent. Otherwise, `command`
+        must be an instance of :class:`dict` and will be sent as is.
 
         :Parameters:
-          - `command`: document representing the command to be issued
+          - `command`: document representing the command to be issued,
+            or the name of the command (for simple commands only).
+
+            .. note:: the order of keys in the `command` document is
+               significant (the "verb" must come first), so commands
+               which require multiple keys (eg, `findandmodify`)
+               should use an instance of :class:`~pymongo.son.SON`
+               instead of a Python `dict`.
+
           - `check` (optional): check the response for errors, raising
             :class:`~pymongo.errors.OperationFailure` if there are any
           - `allowable_errors`: if `check` is ``True``, error messages in this
             list will be ignored by error-checking
 
+        .. versionchanged:: 1.5
+           `command` can be a string in addition to a full document.
         .. versionadded:: 1.4
+
+        .. mongodoc:: commands
         """
+
+        if isinstance(command, str):
+            command = {command: 1}
+
         result = self["$cmd"].find_one(command, _sock=_sock,
                                        _must_use_master=True,
                                        _is_command=True)
@@ -284,6 +334,8 @@ class Database(object):
 
         Returns one of (:data:`~pymongo.OFF`,
         :data:`~pymongo.SLOW_ONLY`, :data:`~pymongo.ALL`).
+
+        .. mongodoc:: profiling
         """
         result = self.command({"profile": -1})
 
@@ -299,6 +351,8 @@ class Database(object):
 
         :Parameters:
           - `level`: the profiling level to use
+
+        .. mongodoc:: profiling
         """
         if not isinstance(level, int) or level < 0 or level > 2:
             raise ValueError("level must be one of (OFF, SLOW_ONLY, ALL)")
@@ -307,6 +361,8 @@ class Database(object):
 
     def profiling_info(self):
         """Returns a list containing current profiling information.
+
+        .. mongodoc:: profiling
         """
         return list(self["system.profile"].find())
 
@@ -316,7 +372,7 @@ class Database(object):
         Return None if the last operation was error-free. Otherwise return the
         error that occurred.
         """
-        error = self.command({"getlasterror": 1})
+        error = self.command("getlasterror")
         if error.get("err", 0) is None:
             return None
         if error["err"] == "not master":
@@ -328,7 +384,7 @@ class Database(object):
 
         Returns a SON object with status information.
         """
-        return self.command({"getlasterror": 1})
+        return self.command("getlasterror")
 
     def previous_error(self):
         """Get the most recent error to have occurred on this database.
@@ -337,7 +393,7 @@ class Database(object):
         `Database.reset_error_history`. Returns None if no such errors have
         occurred.
         """
-        error = self.command({"getpreverror": 1})
+        error = self.command("getpreverror")
         if error.get("err", 0) is None:
             return None
         return error
@@ -348,7 +404,7 @@ class Database(object):
         Calls to `Database.previous_error` will only return errors that have
         occurred since the most recent call to this method.
         """
-        self.command({"reseterror": 1})
+        self.command("reseterror")
 
     def __iter__(self):
         return self
@@ -356,18 +412,6 @@ class Database(object):
     def next(self):
         raise TypeError("'Database' object is not iterable")
 
-    def _password_digest(self, username, password):
-        """Get a password digest to use for authentication.
-        """
-        if not isinstance(password, basestring):
-            raise TypeError("password must be an instance of basestring")
-        if not isinstance(username, basestring):
-            raise TypeError("username must be an instance of basestring")
-
-        md5hash = _md5func()
-        md5hash.update(username.encode('utf-8') + ":mongo:" + password.encode('utf-8'))
-        return unicode(md5hash.hexdigest())
-
     def add_user(self, name, password):
         """Create user `name` with password `password`.
 
@@ -383,7 +427,7 @@ class Database(object):
         """
         self.system.users.update({"user": name},
                                  {"user": name,
-                                  "pwd": self._password_digest(name, password)},
+                                  "pwd": helpers._password_digest(name, password)},
                                  upsert=True, safe=True)
 
     def remove_user(self, name):
@@ -439,23 +483,19 @@ class Database(object):
         :Parameters:
           - `name`: the name of the user to authenticate
           - `password`: the password of the user to authenticate
+
+        .. mongodoc:: authenticate
         """
         if not isinstance(name, basestring):
             raise TypeError("name must be an instance of basestring")
         if not isinstance(password, basestring):
             raise TypeError("password must be an instance of basestring")
 
-        result = self.command({"getnonce": 1})
-        nonce = result["nonce"]
-        digest = self._password_digest(name, password)
-        md5hash = _md5func()
-        md5hash.update("%s%s%s" % (nonce, unicode(name), digest))
-        key = unicode(md5hash.hexdigest())
+        nonce = self.command("getnonce")["nonce"]
+        key = helpers._auth_key(nonce, name, password)
         try:
-            result = self.command(SON([("authenticate", 1),
-                                       ("user", unicode(name)),
-                                       ("nonce", nonce),
-                                       ("key", key)]))
+            self.command(SON([("authenticate", 1), ("user", unicode(name)),
+                              ("nonce", nonce), ("key", key)]))
             return True
         except OperationFailure:
             return False
@@ -465,7 +505,7 @@ class Database(object):
 
         Note that other databases may still be authorized.
         """
-        self.command({"logout": 1})
+        self.command("logout")
 
     def dereference(self, dbref):
         """Dereference a DBRef, getting the SON object it points to.
@@ -517,3 +557,51 @@ class Database(object):
         raise TypeError("'Database' object is not callable. If you meant to "
                         "call the '%s' method on a 'Collection' object it is "
                         "failing because no such method exists." % self.__name)
+
+
+class SystemJS(object):
+    """Helper class for dealing with stored JavaScript.
+    """
+
+    def __init__(self, database):
+        """Get a system js helper for the database `database`.
+
+        An instance of :class:`SystemJS` is automatically created for
+        each :class:`Database` instance as :attr:`Database.system_js`,
+        manual instantiation of this class should not be necessary.
+
+        :class:`SystemJS` instances allow for easy manipulation and
+        access to `server-side JavaScript`_:
+
+        .. doctest::
+
+          >>> db.system_js.add1 = "function (x) { return x + 1; }"
+          >>> db.system.js.find({"_id": "add1"}).count()
+          1
+          >>> db.system_js.add1(5)
+          6.0
+          >>> del db.system_js.add1
+          >>> db.system.js.find({"_id": "add1"}).count()
+          0
+
+        .. note:: Requires server version **>= 1.1.1**
+
+        .. versionadded:: 1.5
+
+        .. _server-side JavaScript: http://www.mongodb.org/display/DOCS/Server-side+Code+Execution#Server-sideCodeExecution-Storingfunctionsserverside
+        """
+        # can't just assign it since we've overridden __setattr__
+        object.__setattr__(self, "_database", database)
+
+    def __setattr__(self, name, code):
+        self._database.system.js.save({"_id": name, "value": Code(code)},
+                                       safe=True)
+
+    def __delattr__(self, name):
+        self._database.system.js.remove({"_id": name}, safe=True)
+
+    def __getattr__(self, name):
+        return lambda *args: self._database.eval("function() { return %s."
+                                                 "apply(this, "
+                                                 "arguments); }" % name,
+                                                 *args)
