@@ -1,4 +1,4 @@
-# Copyright 2009 10gen, Inc.
+# Copyright 2009-2010 10gen, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,14 +14,21 @@
 
 """Little bits and pieces used by the driver that don't really fit elsewhere."""
 
-import sys
+try:
+    import hashlib
+    _md5func = hashlib.md5
+except: # for Python < 2.5
+    import md5
+    _md5func = md5.new
 import struct
+import sys
 import warnings
 
-from son import SON
-from errors import OperationFailure, AutoReconnect
-import bson
 import pymongo
+from pymongo import bson
+from pymongo.errors import (OperationFailure,
+                            AutoReconnect)
+from pymongo.son import SON
 
 def _index_list(key_or_list, direction=None):
     """Helper to generate a list of (key, direction) pairs.
@@ -33,6 +40,9 @@ def _index_list(key_or_list, direction=None):
     else:
         if isinstance(key_or_list, basestring):
             return [(key_or_list, pymongo.ASCENDING)]
+        elif not isinstance(key_or_list, list):
+            raise TypeError("if no direction is specified, "
+                            "key_or_list must be an instance of list")
         return key_or_list
 
 
@@ -41,9 +51,13 @@ def _index_document(index_list):
 
     Takes a list of (key, direction) pairs.
     """
-    if not isinstance(index_list, list):
-        raise TypeError("if no direction is specified, key_or_list must be an "
-                        "instance of list")
+    if isinstance(index_list, dict):
+        raise TypeError("passing a dict to sort/create_index/hint is not "
+                        "allowed - use a list of tuples instead. did you "
+                        "mean %r?" % list(index_list.iteritems()))
+    elif not isinstance(index_list, list):
+        raise TypeError("must use a list of (key, direction) pairs, "
+                        "not: %r" % index_list)
     if not len(index_list):
         raise ValueError("key_or_list must not be the empty list")
 
@@ -58,17 +72,6 @@ def _index_document(index_list):
     return index
 
 
-def _reversed(l):
-    """A version of the `reversed()` built-in for Python 2.3.
-    """
-    i = len(l)
-    while i > 0:
-        i -= 1
-        yield l[i]
-if sys.version_info[:3] >= (2, 4, 0):
-    _reversed = reversed
-
-
 def _unpack_response(response, cursor_id=None):
     """Unpack a response from the database.
 
@@ -106,6 +109,29 @@ def _unpack_response(response, cursor_id=None):
     return result
 
 
+def _password_digest(username, password):
+    """Get a password digest to use for authentication.
+    """
+    if not isinstance(password, basestring):
+        raise TypeError("password must be an instance of basestring")
+    if not isinstance(username, basestring):
+        raise TypeError("username must be an instance of basestring")
+
+    md5hash = _md5func()
+    md5hash.update("%s:mongo:%s" % (username.encode('utf-8'),
+                                    password.encode('utf-8')))
+    return unicode(md5hash.hexdigest())
+
+
+def _auth_key(nonce, username, password):
+    """Get an auth key to use for authentication.
+    """
+    digest = _password_digest(username, password)
+    md5hash = _md5func()
+    md5hash.update("%s%s%s" % (nonce, unicode(username), digest))
+    return unicode(md5hash.hexdigest())
+
+
 # These two functions are some magic to get values we can use for deprecating
 # method style access in favor of property style access while remaining
 # backwards compatible.
