@@ -1,4 +1,4 @@
-# Copyright 2009 10gen, Inc.
+# Copyright 2009-2010 10gen, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -38,12 +38,12 @@ instances.
    Added support for encoding/decoding datetimes and regular expressions.
 """
 
-import datetime
 import calendar
+import datetime
 import re
 
-from objectid import ObjectId
-from dbref import DBRef
+from pymongo.dbref import DBRef
+from pymongo.objectid import ObjectId
 
 # TODO support Binary and Code
 # Binary and Code are tricky because they subclass str so json thinks it can
@@ -80,7 +80,7 @@ def default(obj):
         # TODO share this code w/ bson.py?
         millis = int(calendar.timegm(obj.timetuple()) * 1000 +
                      obj.microsecond / 1000)
-        return {"$date": str(millis)}
+        return {"$date": millis}
     if isinstance(obj, _RE_TYPE):
         flags = ""
         if obj.flags & re.IGNORECASE:
