@@ -1,4 +1,4 @@
-# Copyright 2009 10gen, Inc.
+# Copyright 2009-2010 10gen, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -22,15 +22,14 @@ MongoDB.
 .. versionadded:: 1.1.2
 """
 
-import threading
-import struct
 import random
+import struct
 import sys
+import threading
 
-import bson
-
+from pymongo import bson
 try:
-    import _cbson
+    from pymongo import _cbson
     _use_c = True
 except ImportError:
     _use_c = False
@@ -111,6 +110,8 @@ def query(options, collection_name,
     if field_selector is not None:
         data += bson.BSON.from_dict(field_selector)
     return __pack_message(2004, data)
+if _use_c:
+    query = _cbson._query_message
 
 
 def get_more(collection_name, num_to_return, cursor_id):
@@ -121,6 +122,8 @@ def get_more(collection_name, num_to_return, cursor_id):
     data += struct.pack("<i", num_to_return)
     data += struct.pack("<q", cursor_id)
     return __pack_message(2005, data)
+if _use_c:
+    get_more = _cbson._get_more_message
 
 
 def delete(collection_name, spec, safe):
