@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2009 10gen, Inc.
+# Copyright 2009-2010 10gen, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,27 +16,30 @@
 
 """Test the collection module."""
 
-import warnings
-import unittest
-import re
 import itertools
-import time
+import re
 import sys
-sys.path[0:0] = [""]
+import time
+import unittest
+import warnings
 
 from nose.plugins.skip import SkipTest
 
-import qcheck
-from test_connection import get_connection
-import version
-from pymongo.objectid import ObjectId
-from pymongo.code import Code
-from pymongo.binary import Binary
-from pymongo.collection import Collection
-from pymongo.errors import InvalidName, OperationFailure, InvalidDocument
-from pymongo.errors import DuplicateKeyError
+sys.path[0:0] = [""]
+
 from pymongo import ASCENDING, DESCENDING
+from pymongo.binary import Binary
+from pymongo.code import Code
+from pymongo.collection import Collection
+from pymongo.errors import (DuplicateKeyError,
+                            InvalidDocument,
+                            InvalidName,
+                            OperationFailure)
+from pymongo.objectid import ObjectId
 from pymongo.son import SON
+from test.test_connection import get_connection
+from test import (qcheck,
+                  version)
 
 
 class TestCollection(unittest.TestCase):
@@ -88,6 +91,11 @@ class TestCollection(unittest.TestCase):
             count += 1
         self.assertEqual(count, 3)
 
+        db.test.drop_indexes()
+        ix = db.test.create_index([("hello", DESCENDING),
+                                   ("world", ASCENDING)], name="hello_world")
+        self.assertEquals(ix, "hello_world")
+
         db.test.drop_indexes()
         self.assertEqual(db.system.indexes.find({"ns": u"pymongo_test.test"})
                          .count(), 1)
@@ -124,6 +132,11 @@ class TestCollection(unittest.TestCase):
                          db.test.ensure_index("goodbye"))
         self.assertEqual(None, db.test.ensure_index("goodbye"))
 
+        db.test.drop_indexes()
+        self.assertEqual("foo",
+                         db.test.ensure_index("goodbye", name="foo"))
+        self.assertEqual(None, db.test.ensure_index("goodbye", name="foo"))
+
         db.test.drop_indexes()
         self.assertEqual("goodbye_1",
                          db.test.ensure_index("goodbye"))
@@ -282,9 +295,8 @@ class TestCollection(unittest.TestCase):
         self.assertEqual(db.test.doesnotexist.options(), {})
 
         db.drop_collection("test")
-        options = {"capped": True}
-        db.create_collection("test", options)
-        self.assertEqual(db.test.options(), options)
+        db.create_collection("test", capped=True)
+        self.assertEqual(db.test.options(), {"capped": True})
         db.drop_collection("test")
 
     def test_insert_find_one(self):
@@ -425,6 +437,10 @@ class TestCollection(unittest.TestCase):
         self.assertEqual(doc1["_id"], ids[0])
         self.assertEqual(doc2["_id"], ids[1])
 
+        id = db.test.insert([{"hello": 1}])
+        self.assert_(isinstance(id, list))
+        self.assertEqual(1, len(id))
+
     def test_insert_iterables(self):
         db = self.db
 
@@ -476,14 +492,18 @@ class TestCollection(unittest.TestCase):
         db.test.insert({"_id": 1, "x": 1}, safe=True)
         db.test.insert({"_id": 2, "x": 2}, safe=True)
 
-        self.assertRaises(DuplicateKeyError,
+        expected_error = OperationFailure
+        if version.at_least(db.connection, (1, 3)):
+            expected_error = DuplicateKeyError
+
+        self.assertRaises(expected_error,
                           db.test.insert, {"_id": 1}, safe=True)
-        self.assertRaises(DuplicateKeyError,
+        self.assertRaises(expected_error,
                           db.test.insert, {"x": 1}, safe=True)
 
-        self.assertRaises(DuplicateKeyError,
+        self.assertRaises(expected_error,
                           db.test.save, {"x": 2}, safe=True)
-        self.assertRaises(DuplicateKeyError,
+        self.assertRaises(expected_error,
                           db.test.update, {"x": 1}, {"$inc": {"x": 1}}, safe=True)
 
 
@@ -595,7 +615,7 @@ class TestCollection(unittest.TestCase):
     def test_safe_remove(self):
         db = self.db
         db.drop_collection("test")
-        db.create_collection("test", {"capped": True, "size": 1000})
+        db.create_collection("test", capped=True, size=1000)
 
         db.test.insert({"x": 1})
         self.assertEqual(1, db.test.count())
@@ -875,6 +895,17 @@ class TestCollection(unittest.TestCase):
         self.assertEqual(1, self.db.test.find({"query": {"$ne": None}}).count())
         self.assertEqual(1, len(list(self.db.test.find({"query": {"$ne": None}}))))
 
+    def test_min_query(self):
+        self.db.drop_collection("test")
+        self.db.test.save({"x": 1})
+        self.db.test.save({"x": 2})
+        self.db.test.create_index("x")
+
+        self.assertEqual(1, len(list(self.db.test.find({"$min": {"x": 2},
+                                                        "$query": {}}))))
+        self.assertEqual(2, self.db.test.find({"$min": {"x": 2},
+                                               "$query": {}})[0]["x"])
+
     def test_insert_large_document(self):
         self.assertRaises(InvalidDocument, self.db.test.insert,
                           {"foo": "x" * 4 * 1024 * 1024})
